##############################################################################
#    Copyright (C) 2002-03  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import os, urllib, zLOG, string, sys
from Globals import InitializeClass, Persistent, package_home
from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName, UniqueObject
import Products.CMFCore.CMFCorePermissions
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.WorkflowCore import WorkflowAction
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.ActionInformation import ActionInformation
from Products.CMFCore.Expression import Expression
from Products.PageTemplates import PageTemplateFile

# Import permission names
from Products.CMFCore import CMFCorePermissions

from utils import getUniqueId

_zptdir = os.path.join( package_home( globals() ), 'www' )

class TypeFilterTool(UniqueObject, SimpleItem, ActionProviderBase):
    """
        A simple tool for defining what portal types members can add.
        The members can also define which of these types they want to
        add and have access to these in an easy way.
        The Type Filter tool doesn't exactly prevent the users from adding
        other portal types than the one specified but it gives you
        an easy interface for filtering object type for use in skins etc.
    """

    meta_type='CMF Type Filter'
    id = 'portal_typefilter'
    title = 'Type Filter'
    
    _actions = [ActionInformation(
                id='edit_type_roles'
              , title='Type Filter - Roles'
              , action=Expression(
                text="string:${portal/portal_typefilter/absolute_url}/typefilter_setup_roles")
              , condition=Expression(text="python:'portal_configuration' not in portal.objectIds()")
              , permissions=(CMFCorePermissions.ManagePortal,)
              , category='global'
              , visible=1)
              , ActionInformation(
                id='edit_type_roles'
              , title='Roles'
              , action=Expression(
                text="string:${portal/portal_typefilter/absolute_url}/typefilter_setup_roles")
              , condition=Expression(text="python:'portal_configuration' in portal.objectIds()")
              , permissions=(CMFCorePermissions.ManagePortal,)
              , category='configuration'
              , visible=1)
              , ActionInformation(
                id='edit_type_groups'
              , title='Type Filter - Groups'
              , action=Expression(
                text="string:${portal/portal_typefilter/absolute_url}/typefilter_setup_groups")
              , condition=Expression(text="python:'portal_configuration' not in portal.objectIds()"
                            + "and hasattr(portal.acl_users, 'groups')")
              , permissions=(CMFCorePermissions.ManagePortal,)
              , category='global'
              , visible=1)
              , ActionInformation(
                id='edit_type_groups'
              , title='Groups'
              , action=Expression(
                text="string:${portal/portal_typefilter/absolute_url}/typefilter_setup_groups")
              , condition=Expression(text="python:'portal_configuration' in portal.objectIds()"
                            + "and hasattr(portal.acl_users, 'groups')")
              , permissions=(CMFCorePermissions.ManagePortal,)
              , category='configuration'
              , visible=1)
              , ActionInformation(
                id='edit_type_users'
              , title='Type Filter - Users'
              , action=Expression(
                text="string:${portal/portal_typefilter/absolute_url}/typefilter_setup_users")
              , condition=Expression(text="python:'portal_configuration' not in portal.objectIds()")
              , permissions=(CMFCorePermissions.ManagePortal,)
              , category='global'
              , visible=1)
              , ActionInformation(
                id='edit_type_users'
              , title='Users'
              , action=Expression(
                text="string:${portal/portal_typefilter/absolute_url}/typefilter_setup_users")
              , condition=Expression(text="python:'portal_configuration' in portal.objectIds()")
              , permissions=(CMFCorePermissions.ManagePortal,)
              , category='configuration'
              , visible=1)]


    security = ClassSecurityInfo()
    #security.declareObjectPublic()

    manage_options = (  {'label':'Configuration', 'action':'manage_setTypes'}
                      , {'label':'Overview', 'action':'manage_overview'}
                      , {'label':'Groups', 'action':'manage_setGroupTypes'}
                      , {'label':'Users', 'action':'manage_setUserTypes'}
                     ) + ActionProviderBase.manage_options + SimpleItem.manage_options

    #
    #   ZMI methods
    #
    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_overview' )
    manage_overview = PageTemplateFile.PageTemplateFile( 'explainTypeFilter', _zptdir )

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_setTypes')
    manage_setTypes = PageTemplateFile.PageTemplateFile('typeFilterSetTypes', _zptdir )

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_setGroupTypes')
    manage_setGroupTypes = PageTemplateFile.PageTemplateFile('typeFilterSetGroupTypes', _zptdir)

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_setUserTypes')
    manage_setUserTypes = PageTemplateFile.PageTemplateFile('typeFilterSetUserTypes', _zptdir)

    security.declarePrivate('listActions')
    def listActions(self, info=None):
        """
        Return available actions via tool.
        """
        if info:
            folder = info.folder
        else:
            folder = None
        actions = self._actions
        #zLOG.LOG(self.meta_type, zLOG.INFO, 'Folder is %s' % str(folder))
        #zLOG.LOG(self.meta_type, zLOG.INFO, 'Folders types is %s' % str(self.listFilteredTypes(folder)))
        """
        Uncommented becaused unused at this time
        try:
            for type_info in self.listFilteredTypes(folder):
                actions.append(ActionInformation(
                        id='create_new'
                    , title=type_info.Title()
                    , action=Expression(
                            text='string: ${folder_url}/create_new_form?type_name=%s' % string.replace(type_info.getId(),' ', '+'))
                    , condition=Expression(
                            text='python: folder is object')
                    , permissions=(CMFCorePermissions.ManagePortal,)
                    , category='toolbox'
                    , visible=1
                    ))
        except:
            zLOG.LOG(self.meta_type, zLOG.INFO, 'ListActions: %s - %s' % (sys.exc_info()[0], sys.exc_info()[1]))
        """
        return actions

    #security.declarePublic('index_html')
    #def index_html( self ):
    #    """
    #    call the object
    #    """
    #    return self.__call__()

    #security.declarePublic('getId')
    #def getId( self ):
    #    """
    #    """
    #    return self.id

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_setGlobalTypes')
    def manage_setGlobalTypes(self, REQUEST=None):
        """
        Change the tools properties
        """
        if REQUEST is not None:
            allowed_roles_for_types = {}
            current_types = REQUEST['current_types']
            i = 0
            for type in current_types:
                allowed_roles_for_types[type] = REQUEST.get('type_%d' % i, [])
                i = i + 1
            self._allowed_roles_for_types = allowed_roles_for_types
            self._p_changed = 1
            if not REQUEST.has_key('no_redirect'):
                return self.manage_setTypes(
                    self, REQUEST, manage_tabs_message='Properties changed.')

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_setGlobalGroupTypes')
    def manage_setGlobalGroupTypes(self, REQUEST):
        """
        Change the group settings
        """
        allowed_groups_for_types = {}
        current_types = REQUEST['current_types']
        i = 0
        for type in current_types:
            allowed_groups_for_types[type] = REQUEST.get('type_%d' % i, [])
            i = i + 1
        self._allowed_groups_for_types = allowed_groups_for_types
        self._p_changed = 1
        if not REQUEST.has_key('no_redirect'):
            return self.manage_setGroupTypes(self, REQUEST, manage_tabs_message='Groups changed')

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_setGlobalUserTypes')
    def manage_setGlobalUserTypes(self, REQUEST):
        """
        Change the user settings
        """
        allowed_users_for_types = {}
        current_types = REQUEST['current_types']
        i = 0
        for type in current_types:
            allowed_users_for_types[type] = REQUEST.get('type_%d' % i, [])
            i = i + 1
        self._allowed_users_for_types = allowed_users_for_types
        self._p_changed = 1
        if not REQUEST.has_key('noredirect'):
            return self.manage_setUserTypes(self, REQUEST, manage_tabs_message='Users changed')

    security.declarePrivate('addUserTypes')
    def addUserTypes(self, username, types):
        """
        Change the user settings
        """
        zLOG.LOG(self.meta_type, zLOG.INFO, 'Add user types called. Username: %s, Types: %s' \
        % (username, str(types)))
        if not hasattr(self, '_allowed_users_for_types'):
            self._allowed_users_for_types = {}

        allowed_user_types = self._allowed_users_for_types
        all_types = getToolByName(self, 'portal_types').listTypeInfo()

        for typeinfo in all_types:
            typeid = typeinfo.Title()
            if typeid in types:
                users = []
                if allowed_user_types.has_key(typeid):
                    users = allowed_user_types[typeid]
                if username not in users:
                    users.append(username)
                    allowed_user_types[typeid] = users

        zLOG.LOG(self.meta_type, zLOG.INFO, 'Types after set %s' % str(allowed_user_types))
        self._allowed_users_for_types = allowed_user_types
        self._p_changed = 1

    security.declareProtected(CMFCorePermissions.View, 'setTypes')
    def setTypes(self, allowed_content_types=None, REQUEST=None ):
        """
        Change the instance's properties
        """
        home = self._getHomeFolder()
        if allowed_content_types is not None:
            home.allowed_content_types = allowed_content_types
        if REQUEST is not None:
            REQUEST.RESPONSE.redirect("%s/typefilter_edit_form?portal_status_message=%s" % (self.absolute_url(), "Saved"))

    def _getRoles( self, folder=None ):
        if not folder:
            folder = self._getHomeFolder()
        portal_membership = getToolByName(self, 'portal_membership')
        member = portal_membership.getAuthenticatedMember()
        if not portal_membership.isAnonymousUser():
            localroles = folder.get_local_roles_for_userid(member.getUserName())
        roles = member.getRoles()
        if not portal_membership.isAnonymousUser():
            roles = roles + ('Member',)
        return roles


    def _getHomeFolder( self ):
        # get the portal type tool
        #portal_types = getToolByName(self, 'portal_types')

        # get the portal membership tool
        portal_membership = getToolByName(self, 'portal_membership')

        # get the home folder object (check if the user is allowed to view it)
        home = portal_membership.getHomeFolder(verifyPermission=1)
        return home

    security.declarePublic('listTypes')
    def listTypes( self, folder=None ):
        """
            List type info objects for types which can be added in
            a home folder.
        """
        portal_types = getToolByName(self, 'portal_types')
        membership = getToolByName(self, 'portal_membership')
        roles = self._getRoles(folder)
        username = membership.getAuthenticatedMember().getUserName()
        try:
            groupnames = membership.getMemberGroups()
        except:
            groupnames = None
        result = []

        if folder:
            # get a type info object for the home folder
            folderType = folder and portal_types.getTypeInfo(folder) or None
            #zLOG.LOG(self.meta_type, zLOG.INFO, 'ListTypes - folderType %s' % str(folderType))
            #homeType = home and portal_types.getTypeInfo(home) or None

            # if there exists a type info object
            if folderType is not None:
                # check if the folder allows creation of the types in the types tool
                for contentType in portal_types.listTypeInfo():
                    allowtype = folderType.allowType(str(contentType.Title()))
                    allowglobal = self.allowGlobalType(contentType.Title()
                                                                        , role=roles
                                                                        , username=username
                                                                        , groupname=groupnames)
                    if allowtype and allowglobal:
                        #zLOG.LOG(self.meta_type + ' listTypes', zLOG.INFO, 'ListTypes - allowing contentType %s' % str(contentType))
                        result.append( contentType )
            else: # use all types
                #zLOG.LOG(self.meta_type + ' listTypes:', 0, 'No TypeInfo for: %s' % repr(folder))
                for contentType in portal_types.listTypeInfo():
                    if self.allowGlobalType( contentType.Title(), role=roles, username=username, groupname=groupnames):
                        result.append( contentType )
            allowed_types = folder.allowedContentTypes()
            #zLOG.LOG(self.meta_type + ' listTypes:', zLOG.INFO, 'allowed_types %s' % str(allowed_types))
            for r in result:
                if r not in allowed_types:
                    #zLOG.LOG(self.meta_type + ' listTypes:', zLOG.INFO, '%s not in %s' % (r,str(allowed_types)))
                    result.remove(r)
            #result = filter( lambda typ, container=folder:
            #                typ.isConstructionAllowed( container )
            #            , result )
            #zLOG.LOG(self.meta_type + ' listTypes:', zLOG.INFO, 'Types: %s' % str(result))
            return result
        else:
            for contentType in portal_types.listTypeInfo():
                if self.allowGlobalType(contentType.Title(), role=roles, username=username, groupname=groupnames):
                    #zLOG.LOG(self.meta_type + ' listTypes', zLOG.INFO, 'ListTypes - allowing contentType %s' % str(contentType))
                    result.append( contentType )
            return result

    security.declarePublic('listFilteredTypes')
    def listFilteredTypes( self, folder=None ):
        """
            List type info objects for types which can be added in
            a home folder.
        """
        home = self._getHomeFolder()
        result = []
        if not hasattr(home, 'allowed_content_types'):
            typos = self.listTypes(folder)
            #zLOG.LOG('TypeFilter.listFilteredTypes:', zLOG.INFO, 'We dont have any local allowed types.\nReturning: %s' % str(typos))
            return typos
        else:
            for contentType in self.listTypes():
                if self.allowType( contentType.Title()):
                    result.append( contentType )
            #zLOG.LOG('TypeFilter.listFilteredTypes:', zLOG.INFO, 'testing local and global: %d items' % len(result))
            #zLOG.LOG('TypeFilter.listFilteredTypes:', zLOG.INFO, 'Types: %s' % str(result))
            return tuple(result)

    security.declarePublic('getNewId')
    def getNewId( self ):
        """
            Return a new unique ID
        """
        return getUniqueId()

    security.declarePublic('hasOwnSettings')
    def hasOwnSettings( self ):
        """
            Return true if there are local user settings
        """
        home = self._getHomeFolder()
        return hasattr(home, 'allowed_content_types')

    security.declarePublic('allowType')
    def allowType( self, type ):
        """
            Return true if this toolbox should contain type
        """
        home = self._getHomeFolder()
        if not hasattr(home, 'allowed_content_types'):
            return 1

        #zLOG.LOG('ToolBox.allowType:', 0, '%s in %s' % (type, str(home.allowed_content_types)))

        return (type in home.allowed_content_types)

    security.declarePublic('allowGlobalType')
    def allowGlobalType( self, ptype, role=None, username=None, groupname=None ):
        """
            Return true if this toolbox should contain type
        """
        if not hasattr(self, '_allowed_roles_for_types'):
            self._allowed_roles_for_types = {}
        if not hasattr(self, '_allowed_groups_for_types'):
            self._allowed_groups_for_types = {}
        if not hasattr(self, '_allowed_users_for_types'):
            self._allowed_users_for_types = {}

        roleallowed = 0
        userallowed = 0
        groupallowed = 0
        #zLOG.LOG(self.meta_type+'.allowGlobalType:', 0, 'testing if %s is allowed for %s' % (ptype, str(role)))
        if role is not None:
            if self._allowed_roles_for_types.has_key(ptype):
                if type(role) is type([]) or type(role) is type(()):
                    for r in role:
                        if r in self._allowed_roles_for_types[ptype]:
                            #zLOG.LOG(self.meta_type+'.allowGlobalType:',0,1)
                            roleallowed = 1
                else:
                    roleallowed = role in self._allowed_roles_for_types[ptype]
                    #zLOG.LOG(self.meta_type+'.allowGlobalType:',0,res)

        if username is not None:
            if self._allowed_users_for_types.has_key(ptype):
                userallowed = username in self._allowed_users_for_types[ptype]

        if groupname is not None:
            if self._allowed_groups_for_types.has_key(ptype):
                if type(groupname) is type([]) or type(groupname) is type(()):
                    for g in groupname:
                        if g in self._allowed_groups_for_types[ptype]:
                            groupallowed = 1
                else:
                    groupallowed = groupname in self._allowed_groups_for_types[ptype]
            #zLOG.LOG(self.meta_type+'.allowGlobalType:', 0, '%s in %s' % (ptype, str(self._allowed_roles_for_types.keys())))
        return (roleallowed or userallowed or groupallowed)

InitializeClass(TypeFilterTool)

